
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    print qq{1..0 # SKIP these tests are for testing by the author\n};
    exit
  }
}

use Test2::V1 -ipP;

skip_all 'This test requires Test::Pod::LinkCheck::Lite'
	unless eval { require Test::Pod::LinkCheck::Lite; 1 };

my $t = Test::Pod::LinkCheck::Lite->new;
$t->all_pod_files_ok('lib');

done_testing;
